@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2005, Ben Burton                                   *
+ *  Copyright (c) 1999-2006, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -19,8 +19,8 @@
  *                                                                        *
  *  You should have received a copy of the GNU General Public             *
  *  License along with this program; if not, write to the Free            *
- *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,        *
- *  MA 02111-1307, USA.                                                   *
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,       *
+ *  MA 02110-1301, USA.                                                   *
  *                                                                        *
  **************************************************************************/
 
@@ -42,6 +42,7 @@
 namespace regina {
 
 class NGluingPerms;
+class NGluingPermSearcher;
 class NPacket;
 class NProgressManager;
 class NProgressMessage;
@@ -349,8 +350,7 @@ class NCensus {
          * faces are all internal; if the given triangulation has
          * boundary faces then this routine will simply return \c true.
          *
-         * \ifacespython Parameter \a ignore is not present (and will
-         * be treated as 0).
+         * \ifacespython Parameter \a ignore is not present.
          *
          * @param tri the triangulation to examine.
          * @param ignore a parameter that is ignored.
@@ -358,7 +358,7 @@ class NCensus {
          * non-minimal, or \c true if minimality of the given
          * triangulation has not been determined.
          */
-        static bool mightBeMinimal(NTriangulation* tri, void* ignore);
+        static bool mightBeMinimal(NTriangulation* tri, void* ignore = 0);
 
         /**
          * Fills the given packet with all completions of the given base
@@ -482,11 +482,17 @@ class NCensus {
          * found.  This routine generates the corresponding triangulation
          * and decides whether it really belongs in the census.
          *
+         * \pre The given set of gluing permutations is complete, i.e.,
+         * it is not just a partially-complete search state.  Partial
+         * searches are formed by calling NGluingPermSearcher::runSearch()
+         * with a non-negative depth argument.
+         *
          * @param perms the gluing permutation set that has been found.
          * @param census the census currently being generated;
          * this must really be of class NCensus.
          */
-        static void foundGluingPerms(const NGluingPerms* perms, void* census);
+        static void foundGluingPerms(const NGluingPermSearcher* perms,
+            void* census);
 };
 
 /*@}*/
