@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2005, Ben Burton                                   *
+ *  Copyright (c) 1999-2006, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -19,8 +19,8 @@
  *                                                                        *
  *  You should have received a copy of the GNU General Public             *
  *  License along with this program; if not, write to the Free            *
- *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,        *
- *  MA 02111-1307, USA.                                                   *
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,       *
+ *  MA 02110-1301, USA.                                                   *
  *                                                                        *
  **************************************************************************/
 
@@ -32,6 +32,8 @@
 #include "census/nfacepairing.h"
 #include "triangulation/nfacepair.h"
 #include "triangulation/npermit.h"
+#include "triangulation/ntetrahedron.h"
+#include "triangulation/ntriangulation.h"
 #include "utilities/memutils.h"
 #include "utilities/stringutils.h"
 
@@ -51,10 +53,30 @@ namespace {
 
 NFacePairing::NFacePairing(const NFacePairing& cloneMe) : NThread(),
         nTetrahedra(cloneMe.nTetrahedra),
-        pairs(new NTetFace[cloneMe.nTetrahedra]) {
+        pairs(new NTetFace[cloneMe.nTetrahedra * 4]) {
     std::copy(cloneMe.pairs, cloneMe.pairs + (nTetrahedra * 4), pairs);
 }
 
+NFacePairing::NFacePairing(const NTriangulation& tri) :
+        nTetrahedra(tri.getNumberOfTetrahedra()),
+        pairs(new NTetFace[tri.getNumberOfTetrahedra() * 4]) {
+    unsigned t, f, index;
+    const NTetrahedron *tet, *adj;
+    for (index = 0, t = 0; t < nTetrahedra; t++) {
+        tet = tri.getTetrahedron(t);
+        for (f = 0; f < 4; f++) {
+            adj = tet->getAdjacentTetrahedron(f);
+            if (adj) {
+                pairs[index].tet = tri.getTetrahedronIndex(adj);
+                pairs[index].face = tet->getAdjacentFace(f);
+            } else
+                pairs[index].setBoundary(nTetrahedra);
+
+            index++;
+        }
+    }
+}
+
 std::string NFacePairing::toString() const {
     std::ostringstream ans;
 
@@ -68,6 +90,48 @@ std::string NFacePairing::toString() const {
     return ans.str();
 }
 
+void NFacePairing::writeDotHeader(std::ostream& out, const char* graphName) {
+    static const char defaultGraphName[] = "G";
+
+    if ((! graphName) || (! *graphName))
+        graphName = defaultGraphName;
+
+    out << "graph " << graphName << " {" << std::endl;
+    out << "graph [bgcolor=white];" << std::endl;
+    out << "edge [color=black];" << std::endl;
+    out << "node [shape=circle,style=filled,height=0.15,fixedsize=true,label=\"\"];" << std::endl;
+}
+
+void NFacePairing::writeDot(std::ostream& out, const char* prefix,
+        bool subgraph) const {
+    static const char defaultPrefix[] = "g";
+
+    if ((! prefix) || (! *prefix))
+        prefix = defaultPrefix;
+
+    // We are guaranteed that prefix is a non-empty string.
+
+    if (subgraph)
+        out << "subgraph cluster_" << prefix << " {" << std::endl;
+    else
+        writeDotHeader(out, (prefix + std::string("_graph")).c_str());
+
+    unsigned t;
+    int f;
+    NTetFace adj;
+    for (t = 0; t < nTetrahedra; t++)
+        for (f = 0; f < 4; f++) {
+            adj = dest(t, f);
+            if (adj.isBoundary(nTetrahedra) || adj.tet < static_cast<int>(t) ||
+                    (adj.tet == static_cast<int>(t) && adj.face < f))
+                continue;
+            out << prefix << '_' << t << " -- " << prefix << '_'
+                << adj.tet << ';' << std::endl;
+        }
+
+    out << '}' << std::endl;
+}
+
 std::string NFacePairing::toTextRep() const {
     std::ostringstream ans;
 
@@ -195,7 +259,8 @@ bool NFacePairing::hasBrokenDoubleEndedChain() const {
     // Search for the end edge of the first chain.
     unsigned baseTet;
     unsigned baseFace;
-    for (baseTet = 0; baseTet < nTetrahedra - 1; baseTet++)
+    // Skip the last tetrahedron -- any of the two ends will do.
+    for (baseTet = 0; baseTet + 1 < nTetrahedra; baseTet++)
         for (baseFace = 0; baseFace < 3; baseFace++)
             if (dest(baseTet, baseFace).tet == static_cast<int>(baseTet)) {
                 // Here's a face that matches to the same tetrahedron.
@@ -318,7 +383,8 @@ bool NFacePairing::hasWedgedDoubleEndedChain() const {
     // Search for the end edge of the first chain.
     unsigned baseTet;
     unsigned baseFace;
-    for (baseTet = 0; baseTet < nTetrahedra - 1; baseTet++)
+    // Skip the last tetrahedron -- any of the two ends will do.
+    for (baseTet = 0; baseTet + 1 < nTetrahedra; baseTet++)
         for (baseFace = 0; baseFace < 3; baseFace++)
             if (dest(baseTet, baseFace).tet == static_cast<int>(baseTet)) {
                 // Here's a face that matches to the same tetrahedron.
@@ -408,6 +474,385 @@ bool NFacePairing::hasWedgedDoubleEndedChain(unsigned baseTet,
     return false;
 }
 
+bool NFacePairing::hasOneEndedChainWithStrayBigon() const {
+    // Search for the end edge of the chain.
+    unsigned baseTet;
+    unsigned baseFace;
+    for (baseTet = 0; baseTet < nTetrahedra; baseTet++)
+        for (baseFace = 0; baseFace < 3; baseFace++)
+            if (dest(baseTet, baseFace).tet == static_cast<int>(baseTet)) {
+                // Here's a face that matches to the same tetrahedron.
+                if (hasOneEndedChainWithStrayBigon(baseTet, baseFace))
+                    return true;
+
+                // There's no sense in looking for more
+                // self-identifications in this tetrahedron, since if
+                // there's another (different) one it must be a
+                // one-tetrahedron component (and so not applicable).
+                break;
+            }
+
+    // Nothing found.  Boring.
+    return false;
+}
+
+bool NFacePairing::hasOneEndedChainWithStrayBigon(unsigned baseTet,
+        unsigned baseFace) const {
+    // Follow the chain along and see how far we get.
+    NFacePair bdryFaces =
+        NFacePair(baseFace, dest(baseTet, baseFace).face).complement();
+    unsigned bdryTet = baseTet;
+    followChain(bdryTet, bdryFaces);
+
+    // Here's where we must diverge and create the stray bigon.
+
+    // We cannot glue the working pair of faces to each other.
+    if (dest(bdryTet, bdryFaces.lower()).tet == static_cast<int>(bdryTet))
+        return false;
+
+    // Try each possible direction away from the working faces into the bigon.
+    NFacePair bigonFaces;
+    int bigonTet, farTet, extraTet;
+    NTetFace destFace;
+    unsigned ignoreFace;
+    int i;
+    for (i = 0; i < 2; i++) {
+        destFace = dest(bdryTet,
+            i == 0 ? bdryFaces.lower() : bdryFaces.upper());
+        if (destFace.isBoundary(nTetrahedra))
+            continue;
+        bigonTet = destFace.tet;
+
+        for (ignoreFace = 0; ignoreFace < 4; ignoreFace++) {
+            if (destFace.face == static_cast<int>(ignoreFace))
+                continue;
+            // Look for a bigon running away from tetrahedron
+            // destFace.tet, using the two faces that are *not*
+            // destFace.face or ignoreFace.
+            bigonFaces = NFacePair(destFace.face, ignoreFace).complement();
+
+            farTet = dest(bigonTet, bigonFaces.upper()).tet;
+            if (farTet != bigonTet &&
+                    farTet < static_cast<int>(nTetrahedra) /* non-bdry */ &&
+                    farTet == dest(bigonTet, bigonFaces.lower()).tet) {
+                // We have the bigon!
+                // We know that bdryTet != bigonTet != farTet, and we
+                // can prove that bdryTet != farTet using 4-valency.
+
+                // Ensure that we don't have one of our special exceptions.
+                extraTet = dest(bdryTet,
+                    i == 0 ? bdryFaces.upper() : bdryFaces.lower()).tet;
+                // We know extraTet != bigonTet, since otherwise our
+                // one-ended chain would not have stopped when it did.
+                // We also know extraTet != bdryTet by 4-valency.
+                if (extraTet == farTet ||
+                        extraTet >= static_cast<int>(nTetrahedra) /* bdry */)
+                    return true;
+                if (extraTet == dest(bigonTet, ignoreFace).tet) {
+                    // Could be the special case where extraTet joins to
+                    // all of bdryTet, bigonTet and farTet.
+                    // We already have it joined to bdryTet and bigonTet.
+                    // Check farTet.
+                    if (extraTet != dest(farTet, 0).tet &&
+                            extraTet != dest(farTet, 1).tet &&
+                            extraTet != dest(farTet, 2).tet &&
+                            extraTet != dest(farTet, 3).tet)
+                        return true;
+                } else {
+                    // Could be the special case where extraTet joins
+                    // twice to farTet.  If not, we have the type of
+                    // graph we're looking for.
+                    bigonFaces = NFacePair(
+                        dest(bigonTet, bigonFaces.upper()).face,
+                        dest(bigonTet, bigonFaces.lower()).face).complement();
+                    if (extraTet != dest(farTet, bigonFaces.upper()).tet ||
+                            extraTet != dest(farTet, bigonFaces.lower()).tet)
+                        return true;
+                }
+            }
+        }
+    }
+
+    // Nup.  Nothing found.
+    return false;
+}
+
+bool NFacePairing::hasTripleOneEndedChain() const {
+    // Search for the end edge of the first chain.
+    unsigned baseTet;
+    unsigned baseFace;
+    // Skip the last two tetrahedra -- any of the three chains will do.
+    for (baseTet = 0; baseTet + 2 < nTetrahedra; baseTet++)
+        for (baseFace = 0; baseFace < 3; baseFace++)
+            if (dest(baseTet, baseFace).tet == static_cast<int>(baseTet)) {
+                // Here's a face that matches to the same tetrahedron.
+                if (hasTripleOneEndedChain(baseTet, baseFace))
+                    return true;
+
+                // There's no sense in looking for more
+                // self-identifications in this tetrahedron, since if
+                // there's another (different) one it must be a
+                // one-tetrahedron component (and so not applicable).
+                break;
+            }
+
+    // Nothing found.  Boring.
+    return false;
+}
+
+bool NFacePairing::hasTripleOneEndedChain(unsigned baseTet,
+        unsigned baseFace) const {
+    // Follow the chain along and see how far we get.
+    NFacePair bdryFaces =
+        NFacePair(baseFace, dest(baseTet, baseFace).face).complement();
+    unsigned bdryTet = baseTet;
+    followChain(bdryTet, bdryFaces);
+
+    // Here's where we must diverge and hunt for the other two chains.
+
+    // We cannot glue the working pair of faces to each other.
+    if (dest(bdryTet, bdryFaces.lower()).tet == static_cast<int>(bdryTet))
+        return false;
+
+    NTetFace axis1 = dest(bdryTet, bdryFaces.lower());
+    NTetFace axis2 = dest(bdryTet, bdryFaces.upper());
+    if (axis1.isBoundary(nTetrahedra) || axis2.isBoundary(nTetrahedra))
+        return false;
+
+    // We know axis1.tet != axis2.tet because the chain stopped, but
+    // just in case..
+    if (axis1.tet == axis2.tet)
+        return false;
+
+    // Count the number of other chains coming from axis1 and axis2.
+    int exit1, exit2;
+    NTetFace arrive1, arrive2;
+    int nChains = 1;
+    unsigned newChainTet;
+    NFacePair newChainFaces;
+    for (exit1 = 0; exit1 < 4; exit1++) {
+        if (exit1 == axis1.face)
+            continue;
+        arrive1 = dest(axis1.tet, exit1);
+        if (arrive1.tet == static_cast<int>(bdryTet) ||
+                arrive1.tet == axis1.tet || arrive1.tet == axis2.tet ||
+                arrive1.isBoundary(nTetrahedra))
+            continue;
+
+        for (exit2 = 0; exit2 < 4; exit2++) {
+            if (exit2 == axis2.face)
+                continue;
+            arrive2 = dest(axis2.tet, exit2);
+            if (arrive2.tet != arrive1.tet)
+                continue;
+
+            // We have graph edges from axis1 and axis2 to a common vertex,
+            // which is not part of our original chain and is neither axis1
+            // nor axis2.
+
+            // See if there's a (possibly zero-length) chain we can
+            // follow to a loop.
+            newChainTet = arrive1.tet;
+            newChainFaces = NFacePair(arrive1.face, arrive2.face).complement();
+            followChain(newChainTet, newChainFaces);
+
+            if (dest(newChainTet, newChainFaces.lower()).tet ==
+                    static_cast<int>(newChainTet)) {
+                // Got one!
+                if (++nChains == 3)
+                    return true;
+            }
+        }
+    }
+
+    // Nope.  Not enough chains were found.
+    return false;
+}
+
+bool NFacePairing::hasSingleStar() const {
+    int half[4], all[8];
+
+    unsigned first, second;
+    unsigned f1, f2;
+    int i;
+
+    // Skip the last tetrahedron, since we're already testing every
+    // possibility from both sides.
+    for (first = 0; first + 1 < nTetrahedra; first++) {
+        // All four neighbours must be non-boundary and distinct.
+        for (f1 = 0; f1 < 4; f1++) {
+            half[f1] = dest(first, f1).tet;
+            if (half[f1] >= static_cast<int>(nTetrahedra) /* bdry */)
+                break;
+        }
+        if (f1 < 4)
+            continue;
+
+        std::sort(half, half + 4);
+        if (half[0] == half[1] || half[1] == half[2] || half[2] == half[3])
+            continue;
+
+        // Look for the adjacent neighbour.
+        for (f1 = 0; f1 < 4; f1++) {
+            second = dest(first, f1).tet;
+
+            // Now ensure that all eight faces are non-boundary and distinct.
+            for (f2 = 0; f2 < 4; f2++) {
+                all[f2 + 4] = dest(second, f2).tet;
+                if (all[f2 + 4] >= static_cast<int>(nTetrahedra) /* bdry */)
+                    break;
+            }
+            if (f2 < 4)
+                continue;
+
+            // We have to refresh the first half of the all[] array each
+            // time, since every time we sort all[] we mix the first
+            // tetrahedron's neighbours in with the second tetrahedron's
+            // neighbours.
+            std::copy(half, half + 4, all);
+            std::sort(all, all + 8);
+            for (i = 0; i < 7; i++)
+                if (all[i] == all[i + 1])
+                    break;
+            if (i >= 7)
+                return true;
+        }
+    }
+
+    return false;
+}
+
+bool NFacePairing::hasDoubleStar() const {
+    int all[7];
+
+    unsigned first, second;
+    int f, i;
+
+    // Skip the last tetrahedron, since we're already testing every
+    // possibility from both sides.
+    for (first = 0; first + 1 < nTetrahedra; first++) {
+        // All four neighbours must be non-boundary, and three must be
+        // distinct.
+        for (f = 0; f < 4; f++) {
+            all[f] = dest(first, f).tet;
+            if (all[f] >= static_cast<int>(nTetrahedra) /* bdry */)
+                break;
+        }
+        if (f < 4)
+            continue;
+
+        std::sort(all, all + 4);
+
+        // Find the double edge, and move the three distinct tetrahedra
+        // to the beginning of the array.
+        if (all[0] == all[1] && all[1] != all[2] && all[2] != all[3]) {
+            second = all[0];
+            all[0] = all[3];
+        } else if (all[0] != all[1] && all[1] == all[2] && all[2] != all[3]) {
+            second = all[1];
+            all[1] = all[3];
+        } else if (all[0] != all[1] && all[1] != all[2] && all[2] == all[3]) {
+            second = all[2];
+        } else
+            continue;
+
+        // Now look at the edges coming out from the second tetrahedron.
+        for (f = 0; f < 4; f++) {
+            all[f + 3] = dest(second, f).tet;
+            if (all[f + 3] >= static_cast<int>(nTetrahedra) /* bdry */)
+                break;
+        }
+        if (f < 4)
+            continue;
+
+        // Look for duplicates.  We should only have a single duplicate
+        // pair, this being two copies of first.
+        std::sort(all, all + 7);
+        for (i = 0; i < 6; i++)
+            if (all[i] == all[i + 1]) {
+                if (all[i] != static_cast<int>(first))
+                    break;
+                if (i < 5 && all[i] == all[i + 2])
+                    break;
+            }
+        if (i >= 6)
+            return true;
+    }
+
+    return false;
+}
+
+bool NFacePairing::hasDoubleSquare() const {
+    unsigned t1;
+    NTetFace t2;
+    int join, fa, fb;
+    int adj1 = 0, adj2 = 0;
+    bool found;
+
+    // Skip the last three tetrahedra -- any of the four starting points
+    // will do.
+    for (t1 = 0; t1 + 3 < nTetrahedra; t1++)
+        for (join = 0; join < 4; join++) {
+            t2 = dest(t1, join);
+            if (t2.tet == static_cast<int>(t1) || t2.isBoundary(nTetrahedra))
+                continue;
+
+            // We have distinct t1, t2 adjacent.
+            // Search for double edges leaving t1 and t2 for two new
+            // tetrahedra.
+            found = false;
+            for (fa = 0; fa < 3 && ! found; fa++) {
+                if (fa == join)
+                    continue;
+                adj1 = dest(t1, fa).tet;
+                if (adj1 >= static_cast<int>(nTetrahedra) /* bdry */)
+                    continue;
+                if (adj1 == static_cast<int>(t1) || adj1 == t2.tet)
+                    continue;
+                for (fb = fa + 1; fb < 4; fb++) {
+                    if (fb == join)
+                        continue;
+                    if (adj1 == dest(t1, fb).tet) {
+                        found = true;
+                        break;
+                    }
+                }
+            }
+            if (! found)
+                continue;
+
+            found = false;
+            for (fa = 0; fa < 3 && ! found; fa++) {
+                if (fa == t2.face)
+                    continue;
+                adj2 = dest(t2.tet, fa).tet;
+                if (adj2 >= static_cast<int>(nTetrahedra) /* bdry */)
+                    continue;
+                if (adj2 == static_cast<int>(t1) || adj2 == t2.tet ||
+                        adj2 == adj1)
+                    continue;
+                for (fb = fa + 1; fb < 4; fb++) {
+                    if (fb == t2.face)
+                        continue;
+                    if (adj2 == dest(t2.tet, fb).tet) {
+                        found = true;
+                        break;
+                    }
+                }
+            }
+            if (! found)
+                continue;
+
+            // All we need now is a link between adj1 and adj2.
+            for (fa = 0; fa < 4; fa++)
+                if (dest(adj1, fa).tet == adj2)
+                    return true;
+        }
+
+    // Nothing found.
+    return false;
+}
+
 bool NFacePairing::findAllPairings(unsigned nTetrahedra,
         NBoolSet boundary, int nBdryFaces, UseFacePairing use,
         void* useArgs, bool newThread) {
