@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2005, Ben Burton                                   *
+ *  Copyright (c) 1999-2006, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -19,8 +19,8 @@
  *                                                                        *
  *  You should have received a copy of the GNU General Public             *
  *  License along with this program; if not, write to the Free            *
- *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,        *
- *  MA 02111-1307, USA.                                                   *
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,       *
+ *  MA 02110-1301, USA.                                                   *
  *                                                                        *
  **************************************************************************/
 
@@ -36,6 +36,7 @@
 #define __NFACEPAIRING_H
 #endif
 
+#include <iostream>
 #include <list>
 #include "utilities/nbooleans.h"
 #include "utilities/nthread.h"
@@ -45,6 +46,7 @@ namespace regina {
 
 class NFacePair;
 class NFacePairing;
+class NTriangulation;
 
 /**
  * \weakgroup census
@@ -109,6 +111,11 @@ class NFacePairing : public NThread {
                  decided, the face will be paired to itself. */
 
     public:
+        /**
+         * \name Constructors and Destructors
+         */
+        /*@{*/
+
         /**
          * Creates a new face pairing that is a clone of the given face
          * pairing.
@@ -117,11 +124,34 @@ class NFacePairing : public NThread {
          */
         NFacePairing(const NFacePairing& cloneMe);
 
+        /**
+         * Creates the face pairing of given triangulation.  This is the
+         * face pairing that describes how the tetrahedron faces of the
+         * given triangulation are joined together, as described in the
+         * class notes.
+         *
+         * \pre The given triangulation is not empty.
+         *
+         * @param tri the triangulation whose face pairing should be
+         * constructed.
+         */
+        NFacePairing(const NTriangulation& tri);
+
         /**
          * Deallocates any memory used by this structure.
          */
         virtual ~NFacePairing();
 
+        /*@}*/
+        /**
+         * (end: Constructors and Destructors)
+         */
+
+        /**
+         * \name Basic Queries
+         */
+        /*@{*/
+
         /**
          * Returns the number of tetrahedra whose faces are (potentially)
          * paired in this particular matching.
@@ -202,6 +232,22 @@ class NFacePairing : public NThread {
          */
         bool isUnmatched(unsigned tet, unsigned face) const;
 
+        /**
+         * Determines whether this face pairing is closed.
+         * A closed face pairing has no unmatched faces.
+         */
+        bool isClosed() const;
+
+        /*@}*/
+        /**
+         * (end: Basic Queries)
+         */
+
+        /**
+         * \name Isomorphic Representations
+         */
+        /*@{*/
+
         /**
          * Determines whether this face pairing is in canonical form,
          * i.e., is a minimal representative of its isomorphism class.
@@ -249,6 +295,16 @@ class NFacePairing : public NThread {
          */
         void findAutomorphisms(NFacePairingIsoList& list) const;
 
+        /*@}*/
+        /**
+         * (end: Isomorphic Representations)
+         */
+
+        /**
+         * \name Input and Output
+         */
+        /*@{*/
+
         /**
          * Returns a human-readable representation of this face pairing.
          *
@@ -273,6 +329,63 @@ class NFacePairing : public NThread {
          */
         std::string toTextRep() const;
 
+        /**
+         * Writes the graph corresponding to this face pairing in
+         * the Graphviz DOT language.  Every vertex of this graph
+         * represents a tetrahedron, and every edge represents a pair of
+         * tetrahedron faces that are joined together.  Note that for a
+         * closed triangulation this graph will be entirely 4-valent;
+         * for triangulations with boundary faces, some graph vertices
+         * will have degree three or less.
+         *
+         * The graph can either be written as a complete DOT graph, or
+         * as a clustered subgraph within some larger DOT graph
+         * (according to whether the argument \a subgraph is passed as
+         * \c false or \c true).
+         *
+         * If a complete DOT graph is being written, the output may be
+         * used as a standalone DOT file ready for use with Graphviz.
+         *
+         * If a subgraph is being written, the output will contain a
+         * single \c subgraph section that should be inserted into some
+         * larger DOT file.  Note that the output generated by
+         * writeDotHeader(), followed by one or more subgraphs and then
+         * a closing curly brace will suffice.  The subgraph name will
+         * begin with the string \c cluster_, so that Graphviz identifies
+         * it as a cluster accordingly.
+         *
+         * The argument \a prefix will be prepended to the name of each
+         * graph vertex, and will also be used in the name of the graph
+         * or subgraph.  Using unique prefixes becomes important if you
+         * are calling writeDot() several times to generate several
+         * subgraphs for use in a single DOT file.  If the \a prefix
+         * argument is null or empty then a default prefix will be used.
+         *
+         * Note that this routine generates undirected graphs, not
+         * directed graphs.  The final DOT file should be used with
+         * either the \a neato or \a fdp programs shipped with Graphviz.
+         *
+         * \ifacespython The \a out argument is not present; instead
+         * standard output is assumed.
+         *
+         * @param out the output stream to which to write.
+         * @param prefix a string to prepend to the name of each graph
+         * vertex, and to include in the graph or subgraph name; see
+         * above for details.
+         * @param subgraph \c false if a complete standalone DOT graph
+         * should be output, or \c true if a clustered subgraph should
+         * be output for use in some larger DOT file.
+         *
+         * @see http://www.graphviz.org/
+         */
+        void writeDot(std::ostream& out, const char* prefix = 0,
+            bool subgraph = false) const;
+
+        /*@}*/
+        /**
+         * (end: Input and Output)
+         */
+
         /**
          * Reconstructs a face pairing from a text-based representation.
          * This text-based representation must be in the format produced
@@ -292,20 +405,46 @@ class NFacePairing : public NThread {
         static NFacePairing* fromTextRep(const std::string& rep);
 
         /**
-         * Determines whether this face pairing is closed.
-         * A closed face pairing has no unmatched faces.
+         * Writes header information for a Graphviz DOT file that will
+         * describe the graphs for one or more face pairings.  See
+         * the writeDot() documentation for further information on such
+         * graphs.
+         *
+         * The output will be in the Graphviz DOT language, and will
+         * include appropriate display settings for graphs, edges and
+         * nodes.  The opening brace for a \c graph section of the DOT
+         * file is included.
+         *
+         * This routine may be used with writeDot() to generate a single
+         * DOT file containing the graphs for several different face
+         * pairings.  A complete DOT file can be produced by calling
+         * this routine, then calling writeDot() in subgraph mode for
+         * each face pairing, then outputting a final closing curly brace.
+         *
+         * Note that if you require a DOT file containing the graph for
+         * only a single face pairing, this routine is unnecessary; you
+         * may simply call writeDot() in full graph mode instead.
+         *
+         * This routine is suitable for generating undirected graphs, not
+         * directed graphs.  The final DOT file should be used with
+         * either the \a neato or \a fdp programs shipped with Graphviz.
+         *
+         * \ifacespython The \a out argument is not present; instead
+         * standard output is assumed.
+         *
+         * @param out the output stream to which to write.
+         * @param graphName the name of the graph in the DOT file.
+         * If this is null or empty then a default graph name will be used.
+         *
+         * @see http://www.graphviz.org/
          */
-        bool isClosed() const;
+        static void writeDotHeader(std::ostream& out,
+            const char* graphName = 0);
 
         /**
-         * Determines whether this face pairing contains a triple edge.
-         * A triple edge is where two different tetrahedra are joined
-         * along three of their faces.
-         *
-         * @return \c true if and only if this face pairing contains a
-         * triple edge.
+         * \name Subgraph Queries
          */
-        bool hasTripleEdge() const;
+        /*@{*/
 
         /**
          * Follows a chain as far as possible from the given point.
@@ -352,6 +491,22 @@ class NFacePairing : public NThread {
          */
         void followChain(unsigned& tet, NFacePair& faces) const;
 
+        /**
+         * Determines whether this face pairing contains a triple edge.
+         * A triple edge is where two different tetrahedra are joined
+         * along three of their faces.
+         *
+         * A face pairing containing a triple edge cannot model a closed
+         * minimal irreducible P^2-irreducible 3-manifold triangulation on
+         * more than two tetrahedra.  See "Face pairing graphs and 3-manifold
+         * enumeration", Benjamin A. Burton, J. Knot Theory Ramifications
+         * 13 (2004), 1057--1101.
+         *
+         * @return \c true if and only if this face pairing contains a
+         * triple edge.
+         */
+        bool hasTripleEdge() const;
+
         /**
          * Determines whether this face pairing contains a broken
          * double-ended chain.
@@ -379,6 +534,12 @@ class NFacePairing : public NThread {
          * double-ended chain, i.e., the final two unaccounted faces are
          * not joined together.
          *
+         * A face pairing containing a broken double-ended chain cannot
+         * model a closed minimal irreducible P^2-irreducible 3-manifold
+         * triangulation on more than two tetrahedra.  See "Face pairing
+         * graphs and 3-manifold enumeration", Benjamin A. Burton,
+         * J. Knot Theory Ramifications 13 (2004), 1057--1101.
+         *
          * @return \c true if and only if this face pairing contains a
          * broken double-ended chain that is not part of a complete
          * double-ended chain.
@@ -405,6 +566,13 @@ class NFacePairing : public NThread {
          * faces of these two tetrahedra remain unaccounted for by this
          * structure.
          *
+         * A face pairing containing a one-ended chain with a double handle
+         * cannot model a closed minimal irreducible P^2-irreducible
+         * 3-manifold triangulation on more than two tetrahedra.  See
+         * "Face pairing graphs and 3-manifold enumeration",
+         * Benjamin A. Burton, J. Knot Theory Ramifications 13 (2004),
+         * 1057--1101.
+         *
          * @return \c true if and only if this face pairing contains a
          * one-ended chain with a double handle.
          */
@@ -451,11 +619,146 @@ class NFacePairing : public NThread {
          * length).  See hasTripleEdge() and
          * hasOneEndedChainWithDoubleHandle() for further details.
          *
+         * A face pairing containing a wedged double-ended chain
+         * cannot model a closed minimal irreducible P^2-irreducible
+         * 3-manifold triangulation on more than two tetrahedra.  See
+         * "Enumeration of non-orientable 3-manifolds using face pairing
+         * graphs and union-find", Benjamin A. Burton, math.GT/0604584.
+         *
          * @return \c true if and only if this face pairing contains a
          * wedged double-ended chain.
          */
         bool hasWedgedDoubleEndedChain() const;
 
+        /**
+         * Determines whether this face pairing contains a one-ended
+         * chain with a stray bigon.
+         *
+         * A chain involves a sequence of tetrahedra, each joined to the
+         * next along two faces, and is described in detail in the
+         * documentation for followChain().
+         *
+         * A one-ended chain is a chain in which the first tetrahedron
+         * is also joined to itself along one face (i.e., the underlying
+         * face pairing for a layered solid torus).
+         *
+         * A one-ended chain with a stray bigon describes the following
+         * structure.  We begin with a one-ended chain.  Two new
+         * tetrahedra are added; these are joined to each other along
+         * two pairs of faces, and one of the new tetrahedra is joined
+         * to the end of the one-ended chain.  We then ensure that:
+         * - This configuration is not part of a longer one-ended chain
+         *   that encompasses all of the aforementioned tetrahedra;
+         * - There is no extra tetrahedron that is joined to both the
+         *   two new tetrahedra and the end of the chain;
+         * - There is no extra tetrahedron that is joined to the end of
+         *   the chain along one face and the far new tetrahedron along
+         *   two additional faces (where by "the far new tetrahedron"
+         *   we mean the new tetrahedron that was not originally joined to
+         *   the chain).
+         *
+         * Aside from these constraints, the remaining four tetrahedron faces
+         * (two faces of the far new tetrahedron, one face of the other new
+         * tetrahedron, and one face at the end of the chain) remain
+         * unaccounted for by this structure.
+         *
+         * A face pairing containing a structure of this type
+         * cannot model a closed minimal irreducible P^2-irreducible
+         * 3-manifold triangulation on more than two tetrahedra.  See
+         * "Enumeration of non-orientable 3-manifolds using face pairing
+         * graphs and union-find", Benjamin A. Burton, math.GT/0604584.
+         *
+         * @return \c true if and only if this face pairing contains a
+         * one-ended chain with a stray bigon.
+         */
+        bool hasOneEndedChainWithStrayBigon() const;
+
+        /**
+         * Determines whether this face pairing contains a triple
+         * one-ended chain.
+         *
+         * A chain involves a sequence of tetrahedra, each joined to the
+         * next along two faces, and is described in detail in the
+         * documentation for followChain().
+         *
+         * A one-ended chain is a chain in which the first tetrahedron
+         * is also joined to itself along one face (i.e., the underlying
+         * face pairing for a layered solid torus).
+         *
+         * A triple one-ended chain is created from three one-ended
+         * chains as follows.  Two new tetrahedra are added, and each
+         * one-ended chain is joined to each of the new tetrahedra along
+         * a single face.  The remaining two faces (one from each of the
+         * new tetrahedra) remain unaccounted for by this structure.
+         *
+         * A face pairing containing a triple one-ended chain
+         * cannot model a closed minimal irreducible P^2-irreducible
+         * 3-manifold triangulation on more than two tetrahedra.  See
+         * "Enumeration of non-orientable 3-manifolds using face pairing
+         * graphs and union-find", Benjamin A. Burton, math.GT/0604584.
+         *
+         * @return \c true if and only if this face pairing contains a
+         * triple one-ended chain.
+         */
+        bool hasTripleOneEndedChain() const;
+
+        /**
+         * Determines whether this face pairing contains a single-edged
+         * star.
+         *
+         * A single-edged star involves two tetrahedra that are adjacent
+         * along a single face, where the six remaining faces of these
+         * tetrahedra are joined to six entirely new tetrahedra (so that
+         * none of the eight tetrahedra described in this structure are
+         * the same).
+         *
+         * @return \c true if and only if this face pairing contains a
+         * single-edged star.
+         */
+        bool hasSingleStar() const;
+
+        /**
+         * Determines whether this face pairing contains a double-edged
+         * star.
+         *
+         * A double-edged star involves two tetrahedra that are adjacent
+         * along two separate pairs of faces, where the four remaining
+         * faces of these tetrahedra are joined to four entirely new
+         * tetrahedra (so that none of the six tetrahedra described in
+         * this structure are the same).
+         *
+         * @return \c true if and only if this face pairing contains a
+         * double-edged star.
+         */
+        bool hasDoubleStar() const;
+
+        /**
+         * Determines whether this face pairing contains a double-edged
+         * square.
+         *
+         * A double-edged square involves four distinct tetrahedra that
+         * meet each other as follows.  Two pairs of tetrahedra are
+         * joined along two pairs of faces each.  Then each tetrahedron
+         * is joined along a single face to one tetrahedron of the other
+         * pair.  The four tetrahedron faces not yet joined to anything
+         * (one from each tetrahedron) remain unaccounted for by this
+         * structure.
+         *
+         * @return \c true if and only if this face pairing contains a
+         * double-edged square.
+         */
+        bool hasDoubleSquare() const;
+
+        /*@}*/
+        /**
+         * (end: Subgraph Queries)
+         */
+
+        /**
+         * \name Graph Enumeration
+         */
+        /*@{*/
+
         /**
          * Internal to findAllPairings().  This routine should never be
          * called directly.
@@ -472,6 +775,11 @@ class NFacePairing : public NThread {
          */
         void* run(void* param);
 
+        /*@}*/
+        /**
+         * (end: Graph Enumeration)
+         */
+
         /**
          * Generates all possible face pairings satisfying the given
          * constraints.  Only connected face pairings (pairings in which
@@ -718,6 +1026,46 @@ class NFacePairing : public NThread {
          * wedged double-ended chain as described above.
          */
         bool hasWedgedDoubleEndedChain(unsigned tet, unsigned face) const;
+
+        /**
+         * Internal to hasOneEndedChainWithStrayBigon().  This routine assumes
+         * that the give face of the given tetrahedron is glued to this
+         * same tetrahedron, and attempts to find a one-ended chain with
+         * stray bigon for which this tetrahedron is at the end of the
+         * one-ended chain.
+         *
+         * \pre The given face of the given tetrahedron is paired with
+         * another face of the same tetrahedron under this face pairing.
+         *
+         * @param tet the index in the triangulation of the given
+         * tetrahedron.
+         * @param face the number of the given face in the tetrahedron;
+         * this must be between 0 and 3 inclusive.
+         *
+         * @return \c true if and only if this face pairing contains a
+         * one-ended chain with stray bigon as described above.
+         */
+        bool hasOneEndedChainWithStrayBigon(unsigned tet, unsigned face) const;
+
+        /**
+         * Internal to hasTripleOneEndedChain().  This routine assumes
+         * that the give face of the given tetrahedron is glued to this
+         * same tetrahedron, and attempts to find a triple one-ended
+         * chain for which this tetrahedron is at the end of one of the
+         * individual one-ended chains.
+         *
+         * \pre The given face of the given tetrahedron is paired with
+         * another face of the same tetrahedron under this face pairing.
+         *
+         * @param tet the index in the triangulation of the given
+         * tetrahedron.
+         * @param face the number of the given face in the tetrahedron;
+         * this must be between 0 and 3 inclusive.
+         *
+         * @return \c true if and only if this face pairing contains a
+         * triple one-ended chain as described above.
+         */
+        bool hasTripleOneEndedChain(unsigned tet, unsigned face) const;
 };
 
 /*@}*/
