@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2005, Ben Burton                                   *
+ *  Copyright (c) 1999-2006, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -19,8 +19,8 @@
  *                                                                        *
  *  You should have received a copy of the GNU General Public             *
  *  License along with this program; if not, write to the Free            *
- *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,        *
- *  MA 02111-1307, USA.                                                   *
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,       *
+ *  MA 02110-1301, USA.                                                   *
  *                                                                        *
  **************************************************************************/
 
@@ -53,11 +53,15 @@ NPacket::~NPacket() {
         delete firstTreeChild;
 
     // Fire a packet event and unregister all listeners.
+    // Unregister *before* we fire the event for each listener -- otherwise,
+    // if we have a listener that deletes itself then things could get nasty.
+    // Don't worry about the listener deleting some *other* listener though,
+    // since std::set can cope with erasures in the midst of a set traversal.
     if (listeners.get()) {
         for (std::set<NPacketListener*>::const_iterator it =
                 listeners->begin(); it != listeners->end(); it++) {
-            (*it)->packetToBeDestroyed(this);
             (*it)->packets.erase(this);
+            (*it)->packetToBeDestroyed(this);
         }
     }
 }
@@ -322,6 +326,60 @@ void NPacket::moveToLast() {
     }
 }
 
+void NPacket::sortChildren() {
+    // Run through the packets from largest to smallest, moving each to
+    // the beginning of the child list in turn.
+    NPacket* endpoint = 0;
+    NPacket* current;
+    NPacket* largest;
+
+    while (1) {
+        // Put current at the beginning of the clump of yet-unsorted children.
+        if (! endpoint)
+            current = firstTreeChild;
+        else
+            current = endpoint->nextTreeSibling;
+        if (! current)
+            break;
+
+        // Find the largest amongst the yet-unsorted children.
+        largest = current;
+        current = current->nextTreeSibling;
+        while (current) {
+            if (current->getPacketLabel() > largest->getPacketLabel())
+                largest = current;
+            current = current->nextTreeSibling;
+        }
+
+        // Move the largest to the front of the list.
+        if (firstTreeChild != largest) {
+            // We know that largest has a previous sibling.
+            largest->prevTreeSibling->nextTreeSibling =
+                largest->nextTreeSibling;
+
+            if (largest->nextTreeSibling)
+                largest->nextTreeSibling->prevTreeSibling =
+                    largest->prevTreeSibling;
+            else
+                lastTreeChild = largest->prevTreeSibling;
+
+            firstTreeChild->prevTreeSibling = largest;
+            largest->nextTreeSibling = firstTreeChild;
+            largest->prevTreeSibling = 0;
+            firstTreeChild = largest;
+        }
+
+        if (! endpoint)
+            endpoint = largest;
+    }
+
+    // Fire a packet event.
+    if (listeners.get())
+        for (std::set<NPacketListener*>::const_iterator it = listeners->begin();
+                it != listeners->end(); it++)
+            (*it)->childrenWereReordered(this);
+}
+
 void NPacket::swapWithNextSibling() {
     if (! nextTreeSibling)
         return;
