@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2005, Ben Burton                                   *
+ *  Copyright (c) 1999-2006, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -19,8 +19,8 @@
  *                                                                        *
  *  You should have received a copy of the GNU General Public             *
  *  License along with this program; if not, write to the Free            *
- *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,        *
- *  MA 02111-1307, USA.                                                   *
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,       *
+ *  MA 02110-1301, USA.                                                   *
  *                                                                        *
  **************************************************************************/
 
@@ -671,6 +671,17 @@ class NPacket : public ShareableObject {
          */
         void moveToLast();
 
+        /**
+         * Sorts the immediate children of this packet according to
+         * their packet labels.  Note that this routine is not
+         * recursive (for instance, grandchildren will not be sorted
+         * within each child packet).
+         *
+         * This routine takes quadratic time in the number of
+         * immediate children (and it's slow quadratic at that).
+         */
+        void sortChildren();
+
         /*@}*/
         /**
          * (end: Tree Manipulation)
