@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2005, Ben Burton                                   *
+ *  Copyright (c) 1999-2006, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -19,8 +19,8 @@
  *                                                                        *
  *  You should have received a copy of the GNU General Public             *
  *  License along with this program; if not, write to the Free            *
- *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,        *
- *  MA 02111-1307, USA.                                                   *
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,       *
+ *  MA 02110-1301, USA.                                                   *
  *                                                                        *
  **************************************************************************/
 
@@ -65,7 +65,7 @@ NAugTriSolidTorus* NAugTriSolidTorus::clone() const {
 }
 
 NManifold* NAugTriSolidTorus::getManifold() const {
-    NSFS* ans = new NSFS();
+    NSFSpace* ans = new NSFSpace();
     if (chainType == CHAIN_MAJOR) {
         // Layered solid torus + layered chain.
         ans->insertFibre(2, 1);
@@ -688,11 +688,11 @@ std::ostream& NAugTriSolidTorus::writeCommonName(std::ostream& out,
         }
 
         if (chainType == CHAIN_MAJOR)
-            out << (tex ? "$J_{" : "J(");
+            out << (tex ? "J_{" : "J(");
         else
-            out << (tex ? "$X_{" : "X(");
+            out << (tex ? "X_{" : "X(");
         return out << chainIndex << " | " << params[roles[0]] << ','
-            << params[roles[1]] << (tex ? "}$" : ")");
+            << params[roles[1]] << (tex ? '}' : ')');
     } else {
         // We have three layered solid tori.
         std::pair<long, long> allParams[3];
@@ -724,13 +724,13 @@ std::ostream& NAugTriSolidTorus::writeCommonName(std::ostream& out,
         }
         sort(allParams, allParams + nAllParams);
 
-        out << (tex ? "$A_{" : "A(");
+        out << (tex ? "A_{" : "A(");
         for (i = 0; i < nAllParams; i++) {
             if (i > 0)
                 out << " | ";
             out << allParams[i].first << ',' << allParams[i].second;
         }
-        return out << (tex ? "}$" : ")");
+        return out << (tex ? '}' : ')');
     }
 }
 
