@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2005, Ben Burton                                   *
+ *  Copyright (c) 1999-2006, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -19,8 +19,8 @@
  *                                                                        *
  *  You should have received a copy of the GNU General Public             *
  *  License along with this program; if not, write to the Free            *
- *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,        *
- *  MA 02111-1307, USA.                                                   *
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,       *
+ *  MA 02110-1301, USA.                                                   *
  *                                                                        *
  **************************************************************************/
 
@@ -39,7 +39,9 @@
 
 namespace regina {
 
+class NIsomorphism;
 class NTetrahedron;
+class NTriangulation;
 
 /**
  * \weakgroup subcomplex
@@ -269,6 +271,30 @@ class NLayeredSolidTorus : public NStandardTriangulation {
         NTriangulation* flatten(const NTriangulation* original,
                 int mobiusBandBdry) const;
 
+        /**
+         * Adjusts the details of this layered solid torus according to
+         * the given isomorphism between triangulations.
+         *
+         * The given isomorphism must describe a mapping from \a originalTri
+         * to \a newTri, and this layered solid torus must currently
+         * refer to tetrahedra in \a originalTri.  After this routine is
+         * called this structure will instead refer to the corresponding
+         * tetrahedra in \a newTri (with changes in vertex/face
+         * numbering also accounted for).
+         *
+         * \pre This layered solid torus currently refers to tetrahedra
+         * in \a originalTri, and \a iso describes a mapping from
+         * \a originalTri to \a newTri.
+         *
+         * @param originalTri the triangulation currently referenced by this
+         * layered solid torus.
+         * @param iso the mapping from \a originalTri to \a newTri.
+         * @param newTri the triangulation to be referenced by the updated
+         * layered solid torus.
+         */
+        void transform(const NTriangulation* originalTri,
+                const NIsomorphism* iso, NTriangulation* newTri);
+
         /**
          * Determines if the given tetrahedron forms the base of a
          * layered solid torus within a triangulation.  The torus need
@@ -287,6 +313,36 @@ class NLayeredSolidTorus : public NStandardTriangulation {
         static NLayeredSolidTorus* formsLayeredSolidTorusBase(
             NTetrahedron* tet);
 
+        /**
+         * Determines if the given tetrahedron forms the top level
+         * tetrahedron of a layered solid torus, with the two given
+         * faces of this tetrahedron representing the boundary of the
+         * layered solid torus.
+         *
+         * Note that the two given faces need not be boundary faces in the
+         * overall triangulation.  That is, the layered solid torus may be
+         * a subcomplex of some larger triangulation.  For example, the
+         * two given faces may be joined to some other tetrahedra outside
+         * the layered solid torus or they may be joined to each other.
+         * In fact, they may even extend this smaller layered solid torus
+         * to a larger layered solid torus.
+         *
+         * @param tet the tetrahedron to examine as a potential top
+         * level of a layered solid torus.
+         * @param topFace1 the face number of the given tetrahedron that
+         * should represent the first boundary face of the layered solid
+         * torus.  This should be between 0 and 3 inclusive.
+         * @param topFace2 the face number of the given tetrahedron that
+         * should represent the second boundary face of the layered solid
+         * torus.  This should be between 0 and 3 inclusive, and should
+         * not be equal to \a topFace1.
+         * @return a newly created structure containing details of the
+         * layered solid torus, or \c null if the given tetrahedron with
+         * its two faces do not form the top level of a layered solid torus.
+         */
+        static NLayeredSolidTorus* formsLayeredSolidTorusTop(
+            NTetrahedron* tet, unsigned topFace1, unsigned topFace2);
+
         /**
          * Determines if the given triangulation component forms a
          * layered solid torus in its entirity.
@@ -386,8 +442,8 @@ inline std::ostream& NLayeredSolidTorus::writeName(std::ostream& out) const {
         << meridinalCuts[2] << ')';
 }
 inline std::ostream& NLayeredSolidTorus::writeTeXName(std::ostream& out) const {
-    return out << "$\\mathop{\\rm LST}(" << meridinalCuts[0] << ','
-        << meridinalCuts[1] << ',' << meridinalCuts[2] << ")$";
+    return out << "\\mathop{\\rm LST}(" << meridinalCuts[0] << ','
+        << meridinalCuts[1] << ',' << meridinalCuts[2] << ')';
 }
 inline void NLayeredSolidTorus::writeTextLong(std::ostream& out) const {
     out << "( " << meridinalCuts[0] << ", " << meridinalCuts[1] << ", "
