@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2005, Ben Burton                                   *
+ *  Copyright (c) 1999-2006, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -19,8 +19,8 @@
  *                                                                        *
  *  You should have received a copy of the GNU General Public             *
  *  License along with this program; if not, write to the Free            *
- *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,        *
- *  MA 02111-1307, USA.                                                   *
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,       *
+ *  MA 02110-1301, USA.                                                   *
  *                                                                        *
  **************************************************************************/
 
@@ -197,8 +197,9 @@ class NPerm {
          *
          * @param cloneMe the permutation whose value will be assigned
          * to this permutation.
+         * @return a reference to this permutation.
          */
-        void operator = (const NPerm& cloneMe);
+        NPerm& operator = (const NPerm& cloneMe);
 
         /**
          * Returns the composition of this permutation with the given
@@ -525,8 +526,9 @@ inline void NPerm::setPermCode(unsigned char newCode) {
     code = newCode;
 }
 
-inline void NPerm::operator = (const NPerm& cloneMe) {
+inline NPerm& NPerm::operator = (const NPerm& cloneMe) {
     code = cloneMe.code;
+    return *this;
 }
 
 inline NPerm NPerm::operator *(const NPerm& q) const {
