@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2005, Ben Burton                                   *
+ *  Copyright (c) 1999-2006, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -19,8 +19,8 @@
  *                                                                        *
  *  You should have received a copy of the GNU General Public             *
  *  License along with this program; if not, write to the Free            *
- *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,        *
- *  MA 02111-1307, USA.                                                   *
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,       *
+ *  MA 02110-1301, USA.                                                   *
  *                                                                        *
  **************************************************************************/
 
@@ -38,7 +38,7 @@
 
 const QString ReginaAbout::regBugAddress(PACKAGE_BUGREPORT);
 
-const QString ReginaAbout::regCopyright("Copyright (c) 1999-2005, Ben Burton");
+const QString ReginaAbout::regCopyright("Copyright (c) 1999-2006, Ben Burton");
 
 const QString ReginaAbout::regDataExt(".rga");
 
@@ -48,14 +48,14 @@ const QString ReginaAbout::regDescription(
 const QString ReginaAbout::regName(I18N_NOOP("Regina"));
 
 const QString ReginaAbout::regReleased(
-    I18N_NOOP("Released ") + QDate(2005, 9, 18).toString(Qt::TextDate));
+    I18N_NOOP("Released ") + QDate(2006, 5, 5).toString(Qt::TextDate));
 
 const QString ReginaAbout::regVersion(PACKAGE_VERSION);
 
 const QString ReginaAbout::regWebsite("http://regina.sourceforge.net/");
 
 const char* ReginaAbout::regLicense =
-    "Copyright (c) 1999-2005, Ben Burton\n\n"
+    "Copyright (c) 1999-2006, Ben Burton\n\n"
     "This program is distributed under the terms of the GPL v2.\n\n"
     "CITATION:\n\n"
     "If you find Regina useful in your research, please consider citing it as\n"
@@ -72,13 +72,19 @@ const char* ReginaAbout::regLicense =
     "Public License for more details.\n\n"
     "You should have received a copy of the GNU General Public License along\n"
     "with this program; if not, write to the Free Software Foundation, Inc.,\n"
-    "59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.\n\n"
+    "51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.\n\n"
     "SNAPPEA KERNEL:\n\n"
     "Since version 4.2 (July 2005), Regina includes portions of the\n"
     "SnapPea kernel which it uses for certain geometric calculations.\n"
     "SnapPea is copyright (c) 1991-2005 by Jeff Weeks, who has graciously\n"
     "given permission for it to be distributed under the terms of the GNU\n"
-    "General Public License as described above.\n";
+    "General Public License as described above.\n\n"
+    "ORB KERNEL:\n\n"
+    "Since version 4.3.1 (May 2006), Regina includes snippets of code from\n"
+    "Orb for use in importing and exporting files in Orb / Casson format.\n"
+    "Orb is based on SnapPea (see above) with additional code written by\n"
+    "Damian Heard, who has also given permission for his code to be\n"
+    "distributed under the terms of the GNU General Public License.\n";
 
 ReginaAbout::ReginaAbout(const char* internalName) :
         KAboutData(internalName, regName, regVersion,
@@ -90,6 +96,8 @@ ReginaAbout::ReginaAbout(const char* internalName) :
     addAuthor("Ben Burton", 0, "bab@debian.org");
 
     // Credits:
+    addCredit("Ryan Budney",
+        I18N_NOOP("Code contributions"));
     addCredit("Marc Culler",
         I18N_NOOP("Helpful discussions"));
     addCredit("Dominique Devriese",
@@ -114,6 +122,8 @@ ReginaAbout::ReginaAbout(const char* internalName) :
         I18N_NOOP("Generous support"));
     addCredit("The University of Melbourne, Australia",
         I18N_NOOP("Generous support"));
+    addCredit("The Victorian Partnership for Advanced Computing",
+        I18N_NOOP("Financial support and much-needed CPU cycles"));
     addCredit("Debian GNU/Linux",
         I18N_NOOP("An exceptional working environment"),
         "http://www.debian.org/");
