@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2005, Ben Burton                                   *
+ *  Copyright (c) 1999-2006, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -19,8 +19,8 @@
  *                                                                        *
  *  You should have received a copy of the GNU General Public             *
  *  License along with this program; if not, write to the Free            *
- *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,        *
- *  MA 02111-1307, USA.                                                   *
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,       *
+ *  MA 02110-1301, USA.                                                   *
  *                                                                        *
  **************************************************************************/
 
@@ -29,18 +29,27 @@
 // Regina core includes:
 #include "manifold/nmanifold.h"
 #include "subcomplex/naugtrisolidtorus.h"
+#include "subcomplex/nblockedsfs.h"
+#include "subcomplex/nblockedsfsloop.h"
+#include "subcomplex/nblockedsfspair.h"
+#include "subcomplex/nblockedsfstriple.h"
 #include "subcomplex/nl31pillow.h"
 #include "subcomplex/nlayeredchain.h"
 #include "subcomplex/nlayeredchainpair.h"
 #include "subcomplex/nlayeredlensspace.h"
 #include "subcomplex/nlayeredloop.h"
 #include "subcomplex/nlayeredsolidtorus.h"
+#include "subcomplex/nlayeredsurfacebundle.h"
 #include "subcomplex/npillowtwosphere.h"
+#include "subcomplex/npluggedtorusbundle.h"
 #include "subcomplex/nplugtrisolidtorus.h"
+#include "subcomplex/nsatblock.h"
+#include "subcomplex/nsatregion.h"
 #include "subcomplex/nsnappedball.h"
 #include "subcomplex/nsnappedtwosphere.h"
 #include "subcomplex/nspiralsolidtorus.h"
 #include "subcomplex/nstandardtri.h"
+#include "subcomplex/ntxicore.h"
 #include "triangulation/nisomorphism.h"
 #include "triangulation/npermit.h"
 #include "triangulation/ntriangulation.h"
@@ -63,6 +72,7 @@
 #include <qwhatsthis.h>
 
 using regina::NPacket;
+using regina::NSatRegion;
 using regina::NTriangulation;
 
 NTriCompositionUI::NTriCompositionUI(regina::NTriangulation* packet,
@@ -187,6 +197,7 @@ void NTriCompositionUI::refresh() {
     findLayeredLensSpaces();
     findLayeredLoops();
     findPlugTriSolidTori();
+    findBlockedTriangulations();
 
     // Look for bounded subcomplexes.
     findLayeredSolidTori();
@@ -219,6 +230,7 @@ void NTriCompositionUI::packetToBeDestroyed(regina::NPacket*) {
     // Our current isomorphism test triangulation is about to be
     // destroyed.
     isoTest->setCurrentItem(0); // (i.e., None)
+    comparingTri = 0; // Don't unlisten, the packet destructor will do that.
     updateIsoPanel();
 }
 
@@ -381,6 +393,197 @@ void NTriCompositionUI::findAugTriSolidTori() {
     }
 }
 
+void NTriCompositionUI::describeSatRegion(const NSatRegion& region,
+        QListViewItem* parent) {
+    QListViewItem* details;
+    QListViewItem* annuli;
+
+    regina::NSatBlockSpec spec;
+    regina::NSatAnnulus ann;
+    unsigned long nAnnuli;
+    long a, b;
+    bool ref, back;
+    QString thisAnnulus, adjAnnulus;
+    for (b = region.numberOfBlocks() - 1; b >= 0; b--) {
+        spec = region.block(b);
+        details = new KListViewItem(parent, i18n("Block %1: %2").
+            arg(b).arg(spec.block->getAbbr().c_str()));
+
+        nAnnuli = spec.block->nAnnuli();
+
+        annuli = new KListViewItem(details, i18n("Adjacencies:"));
+
+        for (a = nAnnuli - 1; a >= 0; a--) {
+            thisAnnulus = i18n("Annulus %1/%2").arg(b).arg(a);
+            if (! spec.block->hasAdjacentBlock(a))
+                new KListViewItem(annuli, i18n("%1 --> boundary").
+                    arg(thisAnnulus));
+            else {
+                adjAnnulus = i18n("Annulus %1/%2").
+                    arg(region.blockIndex(spec.block->adjacentBlock(a))).
+                    arg(spec.block->adjacentAnnulus(a));
+                ref = spec.block->adjacentReflected(a);
+                back = spec.block->adjacentBackwards(a);
+
+                if (ref && back)
+                    new KListViewItem(annuli,
+                        i18n("%1 --> %2 (reflected, backwards)").
+                        arg(thisAnnulus).arg(adjAnnulus));
+                else if (ref)
+                    new KListViewItem(annuli, i18n("%1 --> %2 (reflected)").
+                        arg(thisAnnulus).arg(adjAnnulus));
+                else if (back)
+                    new KListViewItem(annuli, i18n("%1 --> %2 (backwards)").
+                        arg(thisAnnulus).arg(adjAnnulus));
+                else
+                    new KListViewItem(annuli, i18n("%1 --> %2").
+                        arg(thisAnnulus).arg(adjAnnulus));
+            }
+        }
+
+        if (nAnnuli == 1)
+            annuli = new KListViewItem(details, i18n("1 annulus"));
+        else
+            annuli = new KListViewItem(details, i18n("%1 annuli").arg(nAnnuli));
+
+        for (a = nAnnuli - 1; a >= 0; a--) {
+            thisAnnulus = i18n("Annulus %1/%2").arg(b).arg(a);
+            ann = spec.block->annulus(a);
+
+            new KListViewItem(annuli,
+                i18n("%1 : Tet %2 (%3%4%5), Tet %6 (%7%8%9)").
+                arg(thisAnnulus).
+                arg(tri->getTetrahedronIndex(ann.tet[0])).
+                arg(ann.roles[0][0]).
+                arg(ann.roles[0][1]).
+                arg(ann.roles[0][2]).
+                arg(tri->getTetrahedronIndex(ann.tet[1])).
+                arg(ann.roles[1][0]).
+                arg(ann.roles[1][1]).
+                arg(ann.roles[1][2]));
+        }
+
+        if (spec.refVert && spec.refHoriz)
+            new KListViewItem(details,
+                i18n("Reflected vertically and horizontally"));
+        else if (spec.refVert)
+            new KListViewItem(details, i18n("Reflected vertically"));
+        else if (spec.refHoriz)
+            new KListViewItem(details, i18n("Reflected horizontally"));
+        else
+            new KListViewItem(details, i18n("No reflections"));
+
+        new KListViewItem(details, spec.block->toString().c_str());
+    }
+}
+
+void NTriCompositionUI::findBlockedTriangulations() {
+    QListViewItem* id;
+    QListViewItem* details;
+
+    regina::NBlockedSFS* sfs = regina::NBlockedSFS::isBlockedSFS(tri);
+    if (sfs) {
+        id = addComponentSection(i18n("Blocked Seifert Fibred Space"));
+        describeSatRegion(sfs->region(), id);
+        delete sfs;
+    }
+
+    regina::NBlockedSFSLoop* loop =
+        regina::NBlockedSFSLoop::isBlockedSFSLoop(tri);
+    if (loop) {
+        id = addComponentSection(i18n("Blocked SFS Loop"));
+
+        details = new KListViewItem(id, i18n("Internal region:"));
+        describeSatRegion(loop->region(), details);
+
+        new KListViewItem(id, i18n("Matching relation: %1").
+            arg(matrixString(loop->matchingReln())));
+
+        delete loop;
+    }
+
+    regina::NBlockedSFSPair* pair =
+        regina::NBlockedSFSPair::isBlockedSFSPair(tri);
+    if (pair) {
+        id = addComponentSection(i18n("Blocked SFS Pair"));
+
+        details = new KListViewItem(id, i18n("Second region:"));
+        describeSatRegion(pair->region(1), details);
+
+        details = new KListViewItem(id, i18n("First region:"));
+        describeSatRegion(pair->region(0), details);
+
+        new KListViewItem(id, i18n("Matching relation (first --> second): %1").
+            arg(matrixString(pair->matchingReln())));
+
+        delete pair;
+    }
+
+    regina::NBlockedSFSTriple* triple =
+        regina::NBlockedSFSTriple::isBlockedSFSTriple(tri);
+    if (triple) {
+        id = addComponentSection(i18n("Blocked SFS Triple"));
+
+        details = new KListViewItem(id, i18n("Second end region:"));
+        describeSatRegion(triple->end(1), details);
+
+        details = new KListViewItem(id, i18n("First end region:"));
+        describeSatRegion(triple->end(0), details);
+
+        details = new KListViewItem(id, i18n("Central region:"));
+        describeSatRegion(triple->centre(), details);
+
+        new KListViewItem(id,
+            i18n("Matching relation (centre --> second end): %1").
+            arg(matrixString(triple->matchingReln(1))));
+
+        new KListViewItem(id,
+            i18n("Matching relation (centre --> first end): %1").
+            arg(matrixString(triple->matchingReln(0))));
+
+        delete triple;
+    }
+
+    regina::NLayeredTorusBundle* bundle =
+        regina::NLayeredTorusBundle::isLayeredTorusBundle(tri);
+    if (bundle) {
+        id = addComponentSection(i18n("Layered Torus Bundle"));
+
+        new KListViewItem(id,
+            i18n("Layering relation (lower a/b --> upper a/b): %1").
+            arg(matrixString(bundle->layeringReln())));
+
+        new KListViewItem(id,
+            i18n("Core relation (upper a/b --> lower a/b): %1").
+            arg(matrixString(bundle->core().parallelReln())));
+
+        new KListViewItem(id,
+            i18n("Core T x I triangulation: %1").
+            arg(bundle->core().getName()));
+
+        delete bundle;
+    }
+
+    regina::NPluggedTorusBundle* pBundle =
+        regina::NPluggedTorusBundle::isPluggedTorusBundle(tri);
+    if (pBundle) {
+        id = addComponentSection(i18n("Plugged Torus Bundle"));
+
+        details = new KListViewItem(id, i18n("Saturated region:"));
+        describeSatRegion(pBundle->region(), details);
+
+        new KListViewItem(id,
+            i18n("Matching relation (joining region boundaries): %1").
+            arg(matrixString(pBundle->matchingReln())));
+
+        new KListViewItem(id,
+            i18n("Thin I-bundle (T x I): %1").
+            arg(pBundle->bundle().getName()));
+
+        delete pBundle;
+    }
+}
+
 void NTriCompositionUI::findL31Pillows() {
     unsigned long nComps = tri->getNumberOfComponents();
 
@@ -788,4 +991,9 @@ QString NTriCompositionUI::edgeString(unsigned long tetIndex,
         arg(roles[endPreimage]);
 }
 
+QString NTriCompositionUI::matrixString(const regina::NMatrix2& matrix) {
+    return QString("[ %1 %2 | %3 %4 ]").
+        arg(matrix[0][0]).arg(matrix[0][1]).arg(matrix[1][0]).arg(matrix[1][1]);
+}
+
 #include "ntricomposition.moc"
