@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2005, Ben Burton                                   *
+ *  Copyright (c) 1999-2006, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -19,8 +19,8 @@
  *                                                                        *
  *  You should have received a copy of the GNU General Public             *
  *  License along with this program; if not, write to the Free            *
- *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,        *
- *  MA 02111-1307, USA.                                                   *
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,       *
+ *  MA 02110-1301, USA.                                                   *
  *                                                                        *
  **************************************************************************/
 
@@ -71,14 +71,24 @@ class NTriSnapPeaUI : public QObject, public PacketViewerTab {
         QLabel* volume;
         NoSnapPea* unavailable;
 
+        /**
+         * Properties
+         */
+        bool allowClosed;
+
     public:
         /**
          * Constructor and destructor.
          */
         NTriSnapPeaUI(regina::NTriangulation* packet,
-            PacketTabbedUI* useParentUI);
+            PacketTabbedUI* useParentUI, bool newAllowClosed);
         ~NTriSnapPeaUI();
 
+        /**
+         * Update properties.
+         */
+        void setAllowClosed(bool newAllowClosed);
+
         /**
          * PacketViewerTab overrides.
          */
@@ -98,4 +108,9 @@ class NTriSnapPeaUI : public QObject, public PacketViewerTab {
         static QString solutionTypeExplanation(int solnType);
 };
 
+inline void NTriSnapPeaUI::setAllowClosed(bool newAllowClosed) {
+    allowClosed = newAllowClosed;
+    refresh();
+}
+
 #endif
