@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2005, Ben Burton                                   *
+ *  Copyright (c) 1999-2006, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -19,8 +19,8 @@
  *                                                                        *
  *  You should have received a copy of the GNU General Public             *
  *  License along with this program; if not, write to the Free            *
- *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,        *
- *  MA 02111-1307, USA.                                                   *
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,       *
+ *  MA 02110-1301, USA.                                                   *
  *                                                                        *
  **************************************************************************/
 
@@ -47,8 +47,8 @@
 #include <kaction.h>
 #include <kfiledialog.h>
 #include <kiconloader.h>
+#include <kinputdialog.h>
 #include <kinstance.h>
-#include <klineeditdlg.h>
 #include <kmainwindow.h>
 #include <kmessagebox.h>
 #include <kstdguiitem.h>
@@ -297,7 +297,7 @@ void ReginaPart::packetRename() {
     bool ok;
     QString suggest = packet->getPacketLabel().c_str();
     while (true) {
-        QString newLabel = KLineEditDlg::getText(i18n("Rename Packet"),
+        QString newLabel = KInputDialog::getText(i18n("Rename Packet"),
             i18n("New label:"), suggest, &ok).stripWhiteSpace();
         if ((! ok) || (newLabel == packet->getPacketLabel().c_str()))
             return;
@@ -355,6 +355,12 @@ void ReginaPart::clonePacket() {
     regina::NPacket* packet = checkPacketSelected();
     if (! packet)
         return;
+    if (! packet->getTreeParent()) {
+        KMessageBox::sorry(widget(), i18n("The root of the packet tree "
+            "cannot be cloned.  You may clone any other packet except for "
+            "this one."));
+        return;
+    }
 
     regina::NPacket* ans = packet->clone(false, false);
 
