@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2005, Ben Burton                                   *
+ *  Copyright (c) 1999-2006, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -19,8 +19,8 @@
  *                                                                        *
  *  You should have received a copy of the GNU General Public             *
  *  License along with this program; if not, write to the Free            *
- *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,        *
- *  MA 02111-1307, USA.                                                   *
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,       *
+ *  MA 02110-1301, USA.                                                   *
  *                                                                        *
  **************************************************************************/
 
@@ -419,7 +419,9 @@ void ReginaMain::fillExamples() {
     fileOpenExample->addURL("closed-hyp-census.rga",
         i18n("Closed Hyperbolic Census"));
     fileOpenExample->addURL("closed-or-census.rga",
-        i18n("Closed Orientable Census"));
+        i18n("Closed Orientable Census (Small)"));
+    fileOpenExample->addURL("closed-or-census-large.rga",
+        i18n("Closed Orientable Census (Large)"));
     fileOpenExample->addURL("closed-nor-census.rga",
         i18n("Closed Non-Orientable Census"));
     fileOpenExample->addURL("snappea-census.rga",
@@ -489,6 +491,9 @@ void ReginaMain::readOptions(KConfig* config) {
         "SpacesPerTab", 4);
     globalPrefs.pythonWordWrap = config->readBoolEntry("WordWrap", false);
 
+    config->setGroup("SnapPea");
+    globalPrefs.snapPeaClosed = config->readBoolEntry("AllowClosed", false);
+
     config->setGroup("Surfaces");
     globalPrefs.surfacesCreationCoords = config->readNumEntry(
         "CreationCoordinates", regina::NNormalSurfaceList::STANDARD);
@@ -515,6 +520,12 @@ void ReginaMain::readOptions(KConfig* config) {
     else
         globalPrefs.triInitialTab = ReginaPrefSet::Gluings; /* default */
 
+    str = config->readEntry("InitialSkeletonTab");
+    if (str == "FacePairingGraph")
+        globalPrefs.triInitialSkeletonTab = ReginaPrefSet::FacePairingGraph;
+    else
+        globalPrefs.triInitialSkeletonTab = ReginaPrefSet::SkelComp; /* def. */
+
     str = config->readEntry("InitialAlgebraTab");
     if (str == "FundGroup")
         globalPrefs.triInitialAlgebraTab = ReginaPrefSet::FundGroup;
@@ -529,6 +540,8 @@ void ReginaMain::readOptions(KConfig* config) {
     config->setGroup("Extensions");
     globalPrefs.triGAPExec = config->readEntry("GAPExec", "gap").
         stripWhiteSpace();
+    globalPrefs.triGraphvizExec = config->readEntry("GraphvizExec", "neato").
+        stripWhiteSpace();
 
     globalPrefs.readPythonLibraries();
 
@@ -567,6 +580,9 @@ void ReginaMain::saveOptions() {
     config->writeEntry("SpacesPerTab", globalPrefs.pythonSpacesPerTab);
     config->writeEntry("WordWrap", globalPrefs.pythonWordWrap);
 
+    config->setGroup("SnapPea");
+    config->writeEntry("AllowClosed", globalPrefs.snapPeaClosed);
+
     config->setGroup("Surfaces");
     config->writeEntry("CreationCoordinates",
         globalPrefs.surfacesCreationCoords);
@@ -594,6 +610,13 @@ void ReginaMain::saveOptions() {
             config->writeEntry("InitialTab", "Gluings"); break;
     }
 
+    switch (globalPrefs.triInitialSkeletonTab) {
+        case ReginaPrefSet::FacePairingGraph:
+            config->writeEntry("InitialSkeletonTab", "FacePairingGraph"); break;
+        default:
+            config->writeEntry("InitialSkeletonTab", "SkelComp"); break;
+    }
+
     switch (globalPrefs.triInitialAlgebraTab) {
         case ReginaPrefSet::FundGroup:
             config->writeEntry("InitialAlgebraTab", "FundGroup"); break;
@@ -608,6 +631,7 @@ void ReginaMain::saveOptions() {
 
     config->setGroup("Extensions");
     config->writeEntry("GAPExec", globalPrefs.triGAPExec);
+    config->writeEntry("GraphvizExec", globalPrefs.triGraphvizExec);
 
     globalPrefs.writePythonLibraries();
 
