@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2005, Ben Burton                                   *
+ *  Copyright (c) 1999-2006, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -19,8 +19,8 @@
  *                                                                        *
  *  You should have received a copy of the GNU General Public             *
  *  License along with this program; if not, write to the Free            *
- *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,        *
- *  MA 02111-1307, USA.                                                   *
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,       *
+ *  MA 02110-1301, USA.                                                   *
  *                                                                        *
  **************************************************************************/
 
@@ -28,11 +28,13 @@
 
 #include "census/nfacepairing.h"
 #include "triangulation/nfacepair.h"
+#include "triangulation/ntriangulation.h"
 #include <boost/python.hpp>
 
 using namespace boost::python;
 using regina::NFacePairing;
 using regina::NTetFace;
+using regina::NTriangulation;
 
 namespace {
     const NTetFace& (NFacePairing::*dest_face)(const NTetFace&) const =
@@ -50,15 +52,33 @@ namespace {
         &NFacePairing::hasOneEndedChainWithDoubleHandle;
     bool (NFacePairing::*query_wdec)() const =
         &NFacePairing::hasWedgedDoubleEndedChain;
+    bool (NFacePairing::*query_oecwsb)() const =
+        &NFacePairing::hasOneEndedChainWithStrayBigon;
+    bool (NFacePairing::*query_toec)() const =
+        &NFacePairing::hasTripleOneEndedChain;
 
     const NTetFace& getItem(const NFacePairing& p, const NTetFace& index) {
         return p[index];
     }
+
+    void writeDot_stdout(const NFacePairing& p, const char* prefix = 0,
+            bool subgraph = false) {
+        p.writeDot(std::cout, prefix, subgraph);
+    }
+
+    void writeDotHeader_stdout(const char* graphName = 0) {
+        NFacePairing::writeDotHeader(std::cout, graphName);
+    }
+
+    BOOST_PYTHON_FUNCTION_OVERLOADS(OL_writeDot, writeDot_stdout, 1, 3);
+    BOOST_PYTHON_FUNCTION_OVERLOADS(OL_writeDotHeader, writeDotHeader_stdout,
+        0, 1);
 }
 
 void addNFacePairing() {
     class_<NFacePairing, std::auto_ptr<NFacePairing>, boost::noncopyable>
             ("NFacePairing", init<const NFacePairing&>())
+        .def(init<const NTriangulation&>())
         .def("getNumberOfTetrahedra", &NFacePairing::getNumberOfTetrahedra)
         .def("dest", dest_face,
             return_value_policy<reference_existing_object>())
@@ -73,6 +93,8 @@ void addNFacePairing() {
         .def("toTextRep", &NFacePairing::toTextRep)
         .def("fromTextRep", &NFacePairing::fromTextRep,
             return_value_policy<manage_new_object>())
+        .def("writeDot", writeDot_stdout, OL_writeDot())
+        .def("writeDotHeader", writeDotHeader_stdout, OL_writeDotHeader())
         .def("isClosed", &NFacePairing::isClosed)
         .def("hasTripleEdge", &NFacePairing::hasTripleEdge)
         .def("followChain", &NFacePairing::followChain)
@@ -83,9 +105,15 @@ void addNFacePairing() {
         .def("hasBrokenDoubleEndedChain", query_bdec)
         .def("hasOneEndedChainWithDoubleHandle", query_oecwdh)
         .def("hasWedgedDoubleEndedChain", query_wdec)
+        .def("hasOneEndedChainWithStrayBigon", query_oecwsb)
+        .def("hasTripleOneEndedChain", query_toec)
 
+        .def("hasSingleStar", &NFacePairing::hasSingleStar)
+        .def("hasDoubleStar", &NFacePairing::hasDoubleStar)
+        .def("hasDoubleSquare", &NFacePairing::hasDoubleSquare)
         .def("__str__", &NFacePairing::toString)
         .staticmethod("fromTextRep")
+        .staticmethod("writeDotHeader")
     ;
 }
 
