@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2005, Ben Burton                                   *
+ *  Copyright (c) 1999-2006, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -19,8 +19,8 @@
  *                                                                        *
  *  You should have received a copy of the GNU General Public             *
  *  License along with this program; if not, write to the Free            *
- *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,        *
- *  MA 02111-1307, USA.                                                   *
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,       *
+ *  MA 02110-1301, USA.                                                   *
  *                                                                        *
  **************************************************************************/
 
@@ -55,12 +55,17 @@ void addNLayeredSolidTorus() {
         .def("getTopFace", &NLayeredSolidTorus::getTopFace)
         .def("flatten", &NLayeredSolidTorus::flatten,
             return_value_policy<manage_new_object>())
+        .def("transform", &NLayeredSolidTorus::transform)
         .def("formsLayeredSolidTorusBase",
             &NLayeredSolidTorus::formsLayeredSolidTorusBase,
             return_value_policy<manage_new_object>())
+        .def("formsLayeredSolidTorusTop",
+            &NLayeredSolidTorus::formsLayeredSolidTorusTop,
+            return_value_policy<manage_new_object>())
         .def("isLayeredSolidTorus", &NLayeredSolidTorus::isLayeredSolidTorus,
             return_value_policy<manage_new_object>())
         .staticmethod("formsLayeredSolidTorusBase")
+        .staticmethod("formsLayeredSolidTorusTop")
         .staticmethod("isLayeredSolidTorus")
     ;
 
