@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2005, Ben Burton                                   *
+ *  Copyright (c) 1999-2006, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -19,22 +19,33 @@
  *                                                                        *
  *  You should have received a copy of the GNU General Public             *
  *  License along with this program; if not, write to the Free            *
- *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,        *
- *  MA 02111-1307, USA.                                                   *
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,       *
+ *  MA 02110-1301, USA.                                                   *
  *                                                                        *
  **************************************************************************/
 
 /* end stub */
 
 void addNAugTriSolidTorus();
+void addNBlockedSFS();
+void addNBlockedSFSLoop();
+void addNBlockedSFSPair();
+void addNBlockedSFSTriple();
 void addNL31Pillow();
 void addNLayeredChain();
 void addNLayeredChainPair();
 void addNLayeredLensSpace();
 void addNLayeredLoop();
 void addNLayeredSolidTorus();
+void addNLayeredSurfaceBundle();
+void addNLayering();
 void addNPillowTwoSphere();
+void addNPluggedTorusBundle();
 void addNPlugTriSolidTorus();
+void addNSatAnnulus();
+void addNSatBlock();
+void addNSatBlockTypes();
+void addNSatRegion();
 void addNSnapPeaCensusTri();
 void addNSnappedBall();
 void addNSnappedTwoSphere();
@@ -42,6 +53,7 @@ void addNSpiralSolidTorus();
 void addNStandardTriangulation();
 void addNTriSolidTorus();
 void addNTrivialTri();
+void addNTxICore();
 
 void addSubcomplex() {
     addNStandardTriangulation();
@@ -52,6 +64,8 @@ void addSubcomplex() {
     addNLayeredLensSpace();
     addNLayeredLoop();
     addNLayeredSolidTorus();
+    addNLayeredSurfaceBundle();
+    addNLayering();
     addNPillowTwoSphere();
     addNPlugTriSolidTorus();
     addNSnapPeaCensusTri();
@@ -60,5 +74,15 @@ void addSubcomplex() {
     addNSpiralSolidTorus();
     addNTriSolidTorus();
     addNTrivialTri();
+    addNTxICore();
+    addNSatAnnulus();
+    addNSatBlock();
+    addNSatBlockTypes();
+    addNSatRegion();
+    addNBlockedSFS();
+    addNBlockedSFSLoop();
+    addNBlockedSFSPair();
+    addNBlockedSFSTriple();
+    addNPluggedTorusBundle();
 }
 
