@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Test Suite                                                            *
  *                                                                        *
- *  Copyright (c) 1999-2005, Ben Burton                                   *
+ *  Copyright (c) 1999-2006, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -19,8 +19,8 @@
  *                                                                        *
  *  You should have received a copy of the GNU General Public             *
  *  License along with this program; if not, write to the Free            *
- *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,        *
- *  MA 02111-1307, USA.                                                   *
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,       *
+ *  MA 02110-1301, USA.                                                   *
  *                                                                        *
  **************************************************************************/
 
@@ -405,6 +405,40 @@ class NSnapPeaTriangulationTest : public CppUnit::TestFixture {
             // testVolume(closedHypNor, "nor_2.02988321", 2.02988121, 7);
         }
 
+        void testZeroVolume(const char* triName, double foundVol,
+                int usePrecision) {
+            // Test whether the given volume is zero to the given number
+            // of decimal places.  If the number of decimal places is
+            // negative, zero places will be used instead.
+
+            if (usePrecision < 0)
+                usePrecision = 0;
+
+            // The trouble here is that we need to take a log to
+            // calculate the output precision for the volume.  If the
+            // volume _is_ zero however, we can't do this.  So we test
+            // first, and only construct the error message if the test
+            // fails.
+            double epsilon = 0.5;
+            for (int i = 0; i < usePrecision; i++)
+                epsilon /= 10;
+
+            if (foundVol <= epsilon && foundVol >= -epsilon)
+                return;
+
+            // FAILURE!  Build the error message and die.
+            int showPrecision =
+                usePrecision + static_cast<int>(ceil(log10(fabs(foundVol))));
+            if (showPrecision < 3)
+                showPrecision = 3;
+
+            std::ostringstream msg;
+            msg << triName << " should have a volume of zero, not "
+                << std::setprecision(showPrecision) << foundVol << '.';
+
+            CPPUNIT_FAIL(msg.str());
+        }
+
         void testFlat(NTriangulation& tri, const char* triName,
                 unsigned places) {
             // Verify that the triangulation has a flat solution and the
@@ -440,20 +474,7 @@ class NSnapPeaTriangulationTest : public CppUnit::TestFixture {
                     precision >= static_cast<int>(places));
             }
 
-            double epsilon = 0.5;
-            for (unsigned i = 0; i < places; i++)
-                epsilon /= 10;
-
-            {
-                std::ostringstream msg;
-                msg << triName << " should have a volume of zero, not "
-                    << std::setprecision(
-                        precision + static_cast<int>(ceil(log10(foundVol))))
-                    << foundVol << '.';
-
-                CPPUNIT_ASSERT_MESSAGE(msg.str(),
-                    foundVol <= epsilon && foundVol >= -epsilon);
-            }
+            testZeroVolume(triName, foundVol, precision);
         }
 
         void flat() {
@@ -462,9 +483,13 @@ class NSnapPeaTriangulationTest : public CppUnit::TestFixture {
         }
 
         void testDegenerate(NTriangulation& tri, const char* triName,
-                unsigned places) {
+                int maxPlaces) {
             // Verify that the triangulation has a degenerate solution
-            // and the volume is zero to the given number of decimal places.
+            // and the volume is zero.  The volume is tested to whatever
+            // precision is reported (up to a maximum of maxPlaces),
+            // but the precision itself has no lower limit imposed -- this
+            // allows flexibility for different floating point behaviours of
+            // different chipsets.
             // Places are counted after the decimal point in standard
             // (non-scientific) notation.
             NSnapPeaTriangulation s(tri);
@@ -487,30 +512,12 @@ class NSnapPeaTriangulationTest : public CppUnit::TestFixture {
 
             int precision;
             double foundVol = s.volume(precision);
-            {
-                std::ostringstream msg;
-                msg << triName << " has a volume with a precision of "
-                    << precision << " places, which is less than the desired "
-                    << places << " places.";
 
-                CPPUNIT_ASSERT_MESSAGE(msg.str(),
-                    precision >= static_cast<int>(places));
-            }
+            // Dumb down the precision to our given maximum.
+            if (precision > maxPlaces)
+                precision = maxPlaces;
 
-            double epsilon = 0.5;
-            for (unsigned i = 0; i < places; i++)
-                epsilon /= 10;
-
-            {
-                std::ostringstream msg;
-                msg << triName << " should have a volume of zero, not "
-                    << std::setprecision(
-                        precision + static_cast<int>(ceil(log10(foundVol))))
-                    << foundVol << '.';
-
-                CPPUNIT_ASSERT_MESSAGE(msg.str(),
-                    foundVol <= epsilon && foundVol >= -epsilon);
-            }
+            testZeroVolume(triName, foundVol, precision);
         }
 
         void degenerate() {
